from pygamefwk.objects.component import Component
from pygamefwk.objects.components.reset import on_reset

def reset():
    SoundListener.listener = None

on_reset.add_lisner(reset)

class SoundListener(Component):
    """소리를 듣는 위치"""
    listener = None

    def __init__(self, object) -> None:
        self.object = object
        if SoundListener.listener != None:
            raise ValueError("이미 SoundListener 컴포넌트는 존재합니다")
        SoundListener.listener = self


import inspect

import pygame
import pygamefwk.util as _util
from pygamefwk.objects import *
from pygamefwk.objects.components.physics import physics_objects

from pygamefwk.manger import Manger
from pygamefwk.sheet import TileSheet, SurfaceSheet

from typing import List, TYPE_CHECKING, Sequence

if TYPE_CHECKING:
    from pygamefwk.objects.gameobject import GameObject
    from pygamefwk.objects.camera import CameraObject

class Scene:

    def __init__(self):
        self.layers: List[List['GameObject']] = [[],[],[],[],[],[],[],[],[],[]]
        self.__display = "main_cam"
        self.obj_cache = {}
        self.camera: 'CameraObject'

    def physics_step(self):
        for phyics in physics_objects:
            phyics.step()

    def start(self):
        for layer in self.layers:
            for obj in layer:
                for component in obj.components:
                    component.start()
                obj.start()

    def update(self):
        """등록된 객체에 update 함수를 실행합니다
        """
        for layer in self.layers:
            for obj in layer:
                for component in obj.components:
                    #try:
                    component.update()
                    #except Exception as e:
                    #    print(f"UPDATE {obj.name} components name: {component.__class__} error {e}")
                #try:
                obj.update()
                #except Exception as e:
                #    print(f"UPDATE {obj.name} error {e}")

    def set_parent(self):
        for layer in self.layers:
            for obj in layer:
                obj.set_parent()

    def render(self, surface: pygame.Surface):
        """등록된 객체에 render 함수를 실행합니다

        Args:
            surface (pygame.Surface): 화면 스크린
        """
        for layer in self.layers:
            for obj in layer:
                if obj.location.world_visible:
                    for component in obj.components:
                        #try:
                        component.render(surface, self.camera)
                        #except Exception as e:
                        #    print(f"RENDER {obj.name} components name: {component.__class__} error {e}")
                    #try:
                    obj.render(surface, self.camera)
                    #except Exception as e:
                    #    print(f"RENDER {obj.name} error {e}")


    
    def add(self, obj: 'GameObject'):
        """
        오브젝트를 추가함
        
        Args:
            obj (GameObject): GameObject 를 상속받은 클래스
        
        """
        layer = self.layers[obj.layer]
        layer.append(obj)
    
    def absorb(self, list : Sequence['GameObject']):
        """
        오브젝트 집합을 추가함
        
        Args:
            list (list): 오브젝트 리스트
        
        """
        for obj in list:
            self.add(obj)
    
    def remove(self, obj: 'GameObject'):
        """오브젝트를 새계에서 삭제하지만 오브젝트 자체는 삭제되지않음"""
        self.layers[obj.layer].remove(obj)
            
    def clear(self):
        """모든 오브젝트를 새계에서 삭제하고 오브젝트 객체도 삭제됨 (delete 함수 실행)
        """
        for layer in self.layers:
            for _ in range(len(layer)):
                layer[0].delete()
    
    def get_object(self, obj_name) -> 'GameObject':
        if obj_name in self.obj_cache:
            return self.obj_cache[obj_name]

        for layer in self.layers:
            for i in range(len(layer)):
                if layer[i].name == obj_name:
                    self.obj_cache[obj_name] = layer[i]
                    return layer[i]
 
    @property
    def display(self):
        return self.__display
    
    @display.setter
    def display(self, value):
        self.__display = value
        self.camera = self.get_object(value)

    def load(self, path: str):
        """오브젝트를 생성하고 새계에 등록합니다

        Args:
            path (str): 멥 파일에 경로 .json

        Raises:
            ImportError: path 경로에 json 에서 class_list 에 존재하지 않는 클래스를 불러오려 할때
            ValueError: 매개변수와 json 에서 저장된 값에 이름이 다르면
        """
        json : dict = _util.replaced_jgetter(_util.jsopen(path))
        setting : dict =json['setting']
        objs =  json['objs']

        tile_len = sum([len(i[2]) for i in setting.get('tile', [])])
        surface_len = sum([len(i[2]) for i in setting.get('surface', [])])

        gameobject_len = sum([len(j) for i in objs for j in list(i.values())])

        loading_length = tile_len + surface_len + gameobject_len

        loading_cnt = 0

        def draw(): #로딩바. 붉은 표면 준비가 느려져서 플래이어를 위한 표시시
            rect = pygame.Rect(10, 10, loading_cnt/loading_length*(Manger.WIDTH-20), 20)
            pygame.draw.rect(Manger.screen, (255, 255, 255), rect)
            pygame.display.update(rect)

        tile_sheet = {}

        for tile_value in setting.get('tile', []):
            tile_sheet[tile_value[0]] = TileSheet(*tile_value)
            loading_cnt += len(tile_value[2])
            draw()

        Manger.tile_sheet = tile_sheet

        surface_sheet = {}

        for surface_value in setting.get('surface', []):
            surface_sheet[surface_value[0]] = SurfaceSheet(*surface_value) 
            loading_cnt += len(surface_value[2])
            draw()

        Manger.surface_sheet = surface_sheet
        
        for key, value in setting.items():
            setattr(Manger, key, value)

        parent_object = ParentObject()
        parent_object.init_instantiate()

        for map_objs in objs:
            for name in map_objs.keys():
                for json_object in map_objs[name]:
                    draw()
                    loading_cnt += 1
                    try:
                        args = list(json_object.values())
                        parameters = list(json_object.keys())
                        prefab_class: 'GameObject' = Manger.classes.get(name, None) # 우선적으로 같은 이름에 게임 오브젝트 클래스를 가져옴옴
                        if prefab_class == None:
                            prefab_class = globals().get(name, None) # pygamefwk 라이브러리 내부에 게임 오브젝트를 가져오는거임임
                            if prefab_class == None:
                                raise ImportError(f"{name} 클레스가 존재하지 않거나 불러지지 않았습니다. \n 현재 불러온 클래스 {Manger.classes}")

                        cls_parameters = list(inspect.signature(prefab_class).parameters.keys())

                        if cls_parameters[-1] == "kwargs":
                            cls_parameters.pop()

                        if cls_parameters == parameters: # 클래스에서 요구하는 파라미터랑 json에서 작성된 값이 같아야 한다. (변수명)
                            prefab: 'GameObject' = prefab_class(*args)
                            if args[0] == "main_cam":
                                self.camera = prefab
                            prefab.init_instantiate()
                            prefab.child_instantiate()
                        else:
                            raise ValueError(f"이름이 틀리거나 순서가 다른것같습니다.\njson :{parameters}\n{name} class:{cls_parameters}")
                    except ValueError as e:
                        print("Value Error message: ", e)
                    except ImportError as e:
                        print("Import Error message: ", e)

        self.set_parent()

        parent_object.location.set_world() # 각 오브젝트 world_position 생성


from pygamefwk.objects.gameobject import GameObject

class MapCenter(GameObject):
    """아무 기능없는 그냥 오브젝트이지만,
    상대좌표로 표기하는 position을 사용해 위치 표기를 더욱 좋게 표기함"""
    def __init__(self, name, position, parent_name):
        super().__init__(name, 1, "parent", True, position, 0, parent_name)


from pygamefwk.objects.component import Component
from typing import TYPE_CHECKING, Dict

if TYPE_CHECKING:
    from pygamefwk.objects.gameobject import GameObject
    from pygamefwk.objects.components.animation import Animation

class AnimationManager(Component):
    def __init__(self, object: 'GameObject', animations: Dict[str, 'Animation'], start_key: str):
        self.object = object
        self.animations = animations
        self.state = start_key
    
    def change_animation(self, animation_name, reset=False):
        if self.state != animation_name:
            self.state = animation_name
            self.animations[self.state].reset()
        else:
            if reset:
                self.animations[self.state].reset()

    def update(self):
        self.animations[self.state].update()


from pygamefwk import util
from pygamefwk.location import Location
from pygamefwk.manger import Manger
from pygamefwk.objects.object import Object
from pygamefwk.vector import Vector
from collections import deque

class GameObject(Object):
    def __init__(self, name, layer, tag, visible, position, rotation, parent_name):
        super().__init__(name, layer, tag)
        self.location = Location(self, Vector(*position), rotation, visible)
        self.parent_name = parent_name
        self.childrens = deque()

    @util.getter
    def render_position(self):
        """프로그램창은 Y좌표가 상반되기에 y좌표를 반전시키는"""
        render_pos = Vector(self.location.world_position.x,  Manger.HEIGHT - self.location.world_position.y)
        return render_pos
    
    def delete(self):
        for child in self.location.children:
            child.object.delete()
        return super().delete()

    def instantiate(self):
        """동적 생성으로 게임이 시작된 이후 실행할떈
        이 함수를 사용해야한다"""
        self.init_instantiate(True)
        self.set_parent()
        self.location.change_location()
        for component in self.components:
            component.start()
        self.start()

    def init_instantiate(self, running=False):
        """게임을 로딩할때 실행되는 함수
        만약 자식 오브젝트를 부모보다 먼저 실행되게 하려면
        children.append(children) 대신 children.init_instantiate() 를 실행하세요"""
        super().init_instantiate()
        self.child_instantiate(running)

    def child_instantiate(self, running=False):
        while self.childrens:
            child: 'GameObject' = self.childrens.popleft()
            if running:
                child.instantiate()
            else:
                child.init_instantiate()

    def set_parent(self):
        """부모 객체와 연결"""
        parent: 'GameObject' = Manger.scene.get_object(self.parent_name)
        self.location.set_parent(parent.location)

    def set_child(self, child):
        self.location.children.append(child)


import pygame
from typing import TYPE_CHECKING, Dict

if TYPE_CHECKING:
    from pygamefwk.scene import Scene
    from pygamefwk.sheet import SurfaceSheet, TileSheet

class Manger:
    """정적 변수를 이곳에 저장함"""
    
    classes = {}
    obj_names = []
    delta_time: float
    tile_sheet: Dict[str, 'TileSheet']
    surface_sheet: Dict[str, 'SurfaceSheet']

    @classmethod
    def init(cls, screen: pygame.Surface, none_scene: 'Scene'):
        """게임에 정적 변수 초기 설정 함수

        Args:
            screen (pygame.Surface): 프로그램 스크린
            none_scene (Scene): 맵
        """
        cls.screen = screen
        cls.scene = none_scene
        cls.WIDTH, cls.HEIGHT = screen.get_size()


from pygame import Rect

from pygamefwk.objects                   import Component, GameObject
from pygamefwk.objects.components.reset  import on_reset
from pygamefwk.event                     import Event
from pygamefwk.vector                    import Vector

from collections import deque
from typing import List

physics_grounds: List[Rect] = []
physics_objects: List['Physics'] = []

def reset():
    physics_grounds.clear()
    physics_objects.clear()

on_reset.add_lisner(reset)

gravity = 0.5

class Physics(Component):
    def __init__(self, object: GameObject, rect: Rect, **kwargs) -> None:
        self.object = object
        self.rect = rect
        self.on_ground = False
        self.friction = 0.45
        self.air_friction = 0.1
        self.velocity = Vector(0, 0)
        self.type: str = kwargs.get('type', 'center')
        physics_objects.append(self)
        self.collision_enter_event = Event()

    def delete(self):
        try:
            physics_objects.remove(self)
        except: pass

    def add_force(self, velocity: Vector):
        self.velocity += velocity

    def step(self):
        self.on_ground = False    
        next_rect = self.rect.copy()
        setattr(next_rect, self.type, self.object.location.position + self.velocity)
        queue = deque()
        for ground in physics_grounds:
            if ground.colliderect(next_rect):
                collide_type = None
                if next_rect.top > ground.bottom - 21:
                    next_rect.top = ground.bottom - 1
                    self.velocity.y = max(self.velocity.y, 0)
                    collide_type = 0 # ground
                    self.on_ground = True
                else:
                    if next_rect.bottom < ground.top + 20:
                        next_rect.bottom = ground.top - 1
                        self.velocity.y = min(self.velocity.y, 0)
                        collide_type = 1 # top
                    elif next_rect.left > ground.right - 15:
                        next_rect.left = ground.right
                        self.velocity.x = min(self.velocity.x, 0)
                        collide_type = 2 # left
                    elif next_rect.right < ground.left + 15:
                        next_rect.right = ground.left
                        self.velocity.x = max(self.velocity.x, 0)
                        collide_type = 3 # right
                queue.append((ground, collide_type))

        while queue:
            ground, collide_type = queue.popleft()
            self.collision_enter_event.invoke(ground, collide_type)

        self.rect = next_rect
        self.object.location.position = getattr(next_rect, self.type)

        self.velocity.y -= gravity

        if self.velocity.y < -20: # 종속 속도
            self.velocity.y += gravity + self.air_friction
        
        if self.velocity.x < -0.2:
            if self.on_ground:
                self.velocity.x += self.friction
            else:
                self.velocity.x += self.air_friction
        elif self.velocity.x > 0.2:
            if self.on_ground:
                self.velocity.x -= self.friction
            else:
                self.velocity.x -= self.air_friction
        else:
            self.velocity.x = 0


from pygame import Surface
from pygamefwk.objects.gameobject import GameObject
from pygamefwk.vector import Vector
from pygamefwk.manger import Manger
from typing import Sequence

class CameraObject(GameObject):
    def __init__(self, name, tag, visible, position, rotation, parent_name):
        super().__init__(name, 0, tag, visible, position, rotation, parent_name)
        self.cam_color = (127, 127, 127)

    def centerXY(self, position: Sequence[float] | Vector):
        centered = Vector(position)
        centered.x += Manger.WIDTH / 2
        centered.y -= Manger.HEIGHT / 2
        return centered

    def __call__(self, position: Sequence[float] | Vector):
        """카메라 시선을 적용한 위치를 반환함

        Args:
            position (tuple[float, float] | Vector): 오브젝트에 위치

        Returns:
            tuple[float, float]: 카메라 시선이 적용된 위치
        """
        camerad = Vector(position)
        rend = self.location.world_position.copy()
        rend.y = Manger.HEIGHT - rend.y # y 좌표만 반전
        camerad = camerad - rend
        camerad.x += Manger.WIDTH / 2 # 0,0 좌표를 프로그램 중앙으로 설정정
        camerad.y += Manger.HEIGHT / 2
        return camerad

    def render(self, surface: Surface, camera):
        surface.fill(self.cam_color) # background render


from pygamefwk.objects.component import Component
from pygamefwk.timertask import TimerTask
from pygamefwk.manger import Manger
from pygamefwk.event import Event

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from pygamefwk.objects.components.image import ImageObject

class Animation(Component):
    """주기적으로 이미지를 변경함"""
    def __init__(self, tick, image_object: 'ImageObject', **kwargs):
        self.object = image_object
        self.period = TimerTask(tick)
        self.index = 0
        self.on_end = Event()
        if 'sheet' in kwargs:
            self.sheet = Manger.surface_sheet[kwargs['sheet']]
            if 'range' in kwargs:
                frist, last = kwargs['range']
                self.images = self.sheet.images[frist:last]
            else:
                self.images = self.sheet.images
        else:
            raise ValueError("키워드가 적절하지 않습니다. sheet 키워드에 사용하는 시트에 이름을 작성하세요")
        self.event_invoked = False
        self.once = kwargs.get('once', False)
        self.len = len(self.images)
        
    def reset(self):
        self.event_invoked = False
        self.period.reset()
        self.pointToIndex(0)

    def pointToIndex(self, index):
        self.index = index
        self.change_image()

    def update(self):
        if self.period.run_periodic_task():
            self.change_image()

    def change_image(self):
        self.object.set_orginal_image(self.images[self.index])
        self.index += 1
        if self.index == self.len:
            if self.once:
                if not self.event_invoked:
                    self.on_end.invoke()
                    self.event_invoked = True
                self.index = self.len -1
            else:
                self.on_end.invoke()
                self.index = 0


from pygamefwk.objects.component import Component
from pygamefwk.objects.object import Object
from pygamefwk.objects.camera import CameraObject
from pygamefwk.objects.parent import ParentObject
from pygamefwk.objects.gameobject import GameObject
from pygamefwk.objects.tilemap import TileMap
from pygamefwk.objects.mapobject import MapCenter
from pygamefwk.objects.ui import *
from pygamefwk.objects.components import *


import json5
from pygamefwk.error import JsonSerializableError
from pygamefwk.error import ImmutableAttributeError

class const(property):
    def settar(self, value):
        raise ImmutableAttributeError("이 속성은 불변입니다 값을 할당하려 하지 마십시오")

class getter(property):
    def settar(self, value):
        raise ImmutableAttributeError("이 속성은 할당할수없습니다 값을 할당하려 하지 마십시오")

def string_insert(string: str, insert_string: str, index: int):
    if len(string) < index or index < 0:
        raise ValueError(f"index 가 문자열에 유효한 위치가 아닙니다 string len: {len(string)} index: {index}")
    return f"{string[:index]}{insert_string}{string[index:]}"

def string_cut(string: str, range: tuple[int, int]):
    if range[0] > range[1]:
        raise ValueError("범위에 끝은 처음보다 크거나 같아야합니다")
    return f"{string[:range[0]]}{string[range[1]:]}"

def jsopen(path: str) -> dict:
    with open(path, 'r') as f:
        json = json5.loads(f.read())
    return json

def is_numeric_array(arr):
    return isinstance(arr, list) and all(isinstance(item, (int, float)) or item is None for item in arr)

def custom_stringify(obj, indent=4, level=0):
    pad = ' ' * (indent * level)

    if isinstance(obj, dict):
        items = []
        for key, value in obj.items():
            item_str = f'{pad}{" " * indent}"{key}": {custom_stringify(value, indent, level + 1)}'
            items.append(item_str)
        return '{\n' + ',\n'.join(items) + f'\n{pad}}}'

    elif isinstance(obj, list):
        if is_numeric_array(obj):
            return '[' + ', '.join('null' if x is None else str(x) for x in obj) + ']'
        else:
            items = [custom_stringify(item, indent, level + 1) for item in obj]
            return '[\n' + ',\n'.join(f'{" " * (indent * (level + 1))}{item}' for item in items) + f'\n{pad}]'

    elif isinstance(obj, str):
        return json5.dumps(obj)

    elif obj is None:
        return 'null'

    else:
        return str(obj)

def jsave(data: dict, path: str):
    with open(path, 'w') as f:
        f.write(custom_stringify(data))

def replaced_jgetter(data):
    if isinstance(data, dict):
        return {key: replaced_jgetter(value) for key, value in data.items()}

    elif isinstance(data, list):
        return [replaced_jgetter(item) for item in data]

    elif isinstance(data, str):
        if data.startswith("gamefs://"):
            path = data.replace("gamefs:/", ".")
            return replaced_jgetter(jsopen(path))
        return data

    else:
        return data

types = (str, int, float, list, bool)

def check_json_serializable(dic: dict):
    for value in list(dic.values()):
        if isinstance(value, types):
            pass
        elif isinstance(value, dict):
            check_json_serializable(value)
        else:
            raise JsonSerializableError(f"{type(value)} 타입은 json 에 저장할수 있는 타입이 아닙니다")




import pygame
from pygame import display
from typing import Callable

from pygamefwk.manger        import Manger
from pygamefwk.scene         import Scene
from pygamefwk.mouse         import mouse_event
from pygamefwk.instantiate   import import_module
from pygamefwk.input         import Input
from pygamefwk.event         import Event
from pygamefwk.objects.components.reset import on_reset

import pygamefwk.mouse as mouse

def world(world_path: str):
    """함수를 새상으로 등록하는 데코레이터
    
    Args:
        world_path (str) : json (맵) 경로
    """
    def real_world(func):
        def wrapper(*args):
            Manger.scene = Scene() # 초기화
            on_reset.invoke()
            Manger.scene.load(world_path) # 맵 불러오기
            start, event, update = func(*args)
            Manger.scene.start()
            start()
            reson = Game.loop(event, update)
            del Manger.scene
            return reson
        return wrapper
    return real_world

event_event = Event()

class Game:
    """게임 엔진"""
    
    @classmethod
    def init(cls, size : tuple[int, int], title : str):
        """게임에 초기 설정

        Args:
            size (tuple[int, int]): 스크린 크기
            title (str): 프로그램에 제목
        """
        pygame.init()
        display.set_caption(title)
        pygame.key.start_text_input()
        Manger.init(display.set_mode(size, pygame.DOUBLEBUF, 32), Scene())
    
    @classmethod
    def import_objects(cls, obj_dir : str, **kwargs):
        """클래스들을 불러와 Manger 에 저장합니다

        Args:
            obj_dir (str): 오브젝트 클래스에 파일를 저장한 폴더 경로
        """
        Manger.classes.update(import_module(obj_dir, **kwargs))
    
    @classmethod
    def stop(cls, reson):
        """맵 종료
        
        Args:
            reson (str): 종료 사유
        """
        cls.reson = reson
        cls.is_running = False
    
    @classmethod
    def loop(cls, events: Callable, func: Callable):
        """맵 루프

        Args:
            events (Callable): 이벤트
            func (Callable): 업데이트
        """
        cls.is_running = True
        cls.is_time_running = True
        cls.clock = pygame.time.Clock()
        cls.reson = None

        while cls.is_running:
            Manger.delta_time = cls.clock.tick(60) / 1000 # fps를 직접적으로 제한

            # 이벤트 체크 시작
            mouse_pressed = mouse.get_pressed()
            
            for i in range(3):
                mouse_click = Input.mouse_click[i]
                if mouse_pressed[i]: #현재 눌림
                    if mouse_click <= Input.KEYUP:
                        Input.mouse_click[i] = Input.KEYDOWN # 방금 누름
                    else:
                        Input.mouse_click[i] = Input.KEYDOWNING # 계속 누름
                else:
                    if mouse_click >= Input.KEYDOWN:
                        Input.mouse_click[i] = Input.KEYUP # 방금땜
                    else:
                        Input.mouse_click[i] = Input.KEYUPING # 계속
                        
            mouse_event()
            
            for key, value in Input.key_board.items():
                if value == 2:
                    Input.key_board[key] = 3
                elif value == 1:
                    Input.key_board[key] = 0
            
            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    cls.stop("quit")
                elif event.type == pygame.KEYDOWN:
                    Input.key_board[event.key] = 2
                elif event.type == pygame.KEYUP:
                    Input.key_board[event.key] = 1
                    
                event_event.invoke(event)
                
                events(event)

            # 이벤트 체크 끝

            func()
            
            if cls.is_time_running:
                Manger.scene.update()

                Manger.scene.physics_step()
            
            Manger.scene.render(Manger.screen)

            pygame.display.update()
        return cls.reson


from pygamefwk.objects.components.image import ImageObject
from pygamefwk.objects.components.physics import Physics, gravity
from pygamefwk.objects.components.animation import Animation
from pygamefwk.objects.components.animationManager import AnimationManager
from pygamefwk.objects.components.soundListener import SoundListener
from pygamefwk.objects.components.soundSource import SoundSource, SoundManager
from pygamefwk.objects.components.reset import on_reset



"""
pygame 의 마우스르 상속받음
"""
from pygamefwk.manger import Manger
from pygamefwk.objects.components.collide_mouse import collide_images
from pygame.mouse import *
from pygamefwk import Vector

def inget_pos():
    pos = Vector(get_pos())
    pos.x -= Manger.WIDTH / 2
    pos.y += Manger.HEIGHT / 2
    return pos

def mouse_event():
    """이미지 오브젝트의 마우스 충돌연산
    """
    for layer in collide_images[::-1]:
        for img_obj in layer[::-1]:
            if img_obj.rect != None and img_obj.object.location.world_visible:
                if img_obj.rect.collidepoint(get_pos()):
                    if img_obj.collide_state == False:
                        img_obj.collide_state = True
                        img_obj.object.on_mouse_enter(get_pos())
                        return 
                    elif img_obj.collide_state == True:
                        img_obj.object.on_mouse_stay(get_pos())
                        return 
                elif img_obj.collide_state == True:
                    img_obj.collide_state = False
                    img_obj.object.on_mouse_exit(get_pos())
                    return 



from pygamefwk.manger import Manger
from pygamefwk.objects.component import Component


class Object(Component):
    def __init__(self, name, layer, tag):
        self.name = name
        self.tag = tag
        self.layer = layer
        self.components : list[Component] = []

    def delete(self): 
        """씬에서 이 오브젝트를 삭제합니다"""
        try:
            for component in self.components:
                component.delete()
                
            Manger.scene.remove(self)
        except: pass
        del self

    def init_instantiate(self):
        Manger.scene.add(self)


import pygame
from pygamefwk.objects.ui import NEWLINE
from pygamefwk.objects.ui.ui import UI
from pygamefwk.vector import Vector

# (path, size): font
class Text(UI):
    """글자를 화면에 나타냄"""
    def __init__(self, name, layer, tag, visible, position, rotation, parent_name, size, color, Font, interval, **kwargs):
        super().__init__(name, layer, tag, visible, position, rotation, parent_name)
        self.Font = Font

        if Font.startswith('./'):
            self.font = pygame.font.Font(Font, size)
        else: 
            self.font = pygame.font.SysFont(Font, size)

        if 'bold' in kwargs:
            self.font.set_bold(kwargs['bold'])

        self.render_type = kwargs.get("render_type", "topleft")

        self.__shadow = kwargs.get("shadow", None)
        self.shadow_pos = [1, 1]

        self.size = size
        self.interval = interval
        self.__color = color
        self.__text = ""
        self.changed = True
        self.render_standard = None
        self.images = None
    
    @property
    def text(self):
        return self.__text
    
    @text.setter
    def text(self, value):
        self.__text = value
        self.changed = True

    @property
    def color(self):
        return self.__color
    
    @color.setter
    def color(self, color):
        self.__color = color
        self.changed = True

    @property
    def shadow(self):
        return self.__shadow
    
    @shadow.setter
    def shadow(self, color):
        self.__shadow = color
        self.changed = True

    def set_position_standard(self, standard):
        self.render_standard = standard

    def get_position(self, index: int) -> Vector:
        """글자에 index 주소로 접근해서 Text 오브젝트 위치를 기준으로 글자에 상대적 위치를 반환

        Args:
            __index (int): 글자 위치

        Returns:
            Vector: position 을 0, 0 으로 할때 x,y 좌표
        """
        line = self.get_line(index)
        text = self.text[:index].split(NEWLINE)[-1]
        x, _ = self.font.size(text)
        y = self.size + self.interval
        y *= line
        return Vector(x-5, -y)

    def get_line(self, index: int) -> int:
        """글자에 index 주소로 접근해서 줄수를 확인함

        Args:
            __index (int): 글자위치

        Returns:
            int: 라인 번호
        """
        line = self.text.count(NEWLINE, 0, index)
        return line + 1

    def render(self, surface : pygame.Surface, camera):
        if self.changed:
            texts = self.text.split(NEWLINE)
            self.images = [self.font.render(text, True, self.__color) for text in texts]
            if self.__shadow != None:
                self.shadow_images = [self.font.render(text, True, self.__shadow) for text in texts]
            if len(self.__color) == 4:
                for image in self.images:
                    image.set_alpha(self.__color[3])
            
            if self.__shadow != None:
                if len(self.__shadow) == 4:
                    for shadow_image in self.shadow_images:
                        shadow_image.set_alpha(self.__shadow[3])
            self.changed = False

        position = self.location.world_position if self.render_standard == 'mouse' else self.render_position
        if self.__shadow != None:
            pos = camera.centerXY(position)
            pos.x += self.shadow_pos[0]
            pos.y += self.shadow_pos[1]
            for image in self.shadow_images:
                rect = image.get_rect()
                setattr(rect, self.render_type, pos)
                pos.y += self.size + self.interval
                surface.blit(image, rect)

        pos = camera.centerXY(position)
        for image in self.images:
            rect = image.get_rect()
            setattr(rect, self.render_type, pos)
            pos.y += self.size + self.interval
            surface.blit(image, rect)



import pygame

from pygamefwk                           import game, util
from pygamefwk.event                     import Event
from pygamefwk.input                     import Input
from pygamefwk.objects.components.image  import ImageObject
from pygamefwk.objects.ui.text           import Text
from pygamefwk.objects.ui.ui             import UI
from pygamefwk.timertask                 import OnceTimerTask, TimerTask

class InputField(UI):
    def __init__(self, name, layer, tag, visible, position, rotation, parent_name, scale, color, font, interval, path, limit, fake_text):
        super().__init__(name, layer, tag, visible, position, rotation, parent_name)

        image = ImageObject(self, path=path, size=(4, 4), type="topleft", follow=True, collide=True)
        self.components.append(image)

        self.input_line = InputLine(name + "_line", layer, tag, False, [0, 0], 0, parent_name, scale)
        self.childrens.append(self.input_line)
        
        self.field = Text(name + '_text', layer, tag + "_text", True, [20, -10], 0, name, scale, color, font, interval)
        self.childrens.append(self.field)

        self.fake = Text(name + '_fake', layer, tag + "_fake", True, [20, -10], 0, name, scale, (128, 128, 128), font, interval)
        self.childrens.append(self.fake)

        self.fake.text = fake_text

        self.text = ""
        self.focused = False
        self.editing_pos = 0
        self.limit = limit

        self.text_edit = False
        self.text_editing = ""
        self.text_editing_pos = 0
        
        self.backspace = False
        self.stay = False
        self.timertask = TimerTask(600)
        self.backtime = TimerTask(40)
        self.wait_backspace = OnceTimerTask(350)
        
        self.input_event = Event()
        game.event_event.add_lisner(self.event)

    def bar_reset(self):
        """커서에 깜빡거림에 주기를 초기화합니다
        """
        self.timertask.reset()
        self.input_line.location.visible = True
        
    def toggle_bar(self):
        """커서를 토글로 껏다킵니다
        """
        self.input_line.location.visible = not self.input_line.location.visible
    
    def toggle_backspace(self):
        """연속 지우기를 토글로 제어합니다
        """
        self.backspace = not self.backspace
    
    def insert(self, index: int, value: str):
        """index 위치에 value 를 삽입합니다

        Args:
            index (int): 위치
            value (str): 삽입할 글자
        """
        self.text = util.string_insert(self.text, value, index)
    
    def cut(self, range: tuple[int, int]):
        """일정 범위에 글자를 잘라냅니다

        Args:
            range (tuple[int, int]): 잘라낼 범위
        """
        self.text = util.string_cut(self.text, range)
    
    def focus_insert(self, value: str):
        """커서를 기준으로 글자를 삽입합니다

        Args:
            value (str): 삽입할 글자
        """
        self.insert(self.editing_pos, value)
        self.set_edit_pos(len(value), add=True)
    
    def focus_cut(self, size: int):
        """커서를 기준으로 글자를 잘라냅니다

        Args:
            size (int): 커서로부터 이만큼 잘라냅니다
        """
        self.cut((self.editing_pos-size, self.editing_pos))
        self.set_edit_pos(size, sub=True)
        
    def set_edit_pos(self, pos: int, **kwargs):
        """커서에 위치를 변경합니다
        add 키워드는 bool 로 True 일떄 인수로 받은 pos를 더합니다
        sub 키워드는 bool 로 True 일떄 인수로 받은 pos를 뺍니다

        Args:
            pos (int): 커서 위치, 또는 연산할 값
        """
        if kwargs.get("add"):
            pos += self.editing_pos
        elif kwargs.get("sub"):
            pos = self.editing_pos - pos
        length = len(self.text+self.text_editing)
        if 0 >= pos:
            self.editing_pos = 0
        elif pos > length:
            self.editing_pos = length
        else:
            self.editing_pos = pos
    
    def on_mouse_enter(self, pos):
        self.stay = True
    
    def on_mouse_stay(self, pos):
        if Input.get_mouse_down(0):
            self.focused = True
    
    def on_mouse_exit(self, pos):
        self.stay = False
    
    def update(self):
        if self.focused:
            if Input.get_key_down(pygame.K_BACKSPACE):
                self.wait_backspace.reset()
                if len(self.text) > 0 and self.editing_pos > 0:
                    self.focus_cut(1)
                    self.bar_reset()   

            elif Input.get_key_down(pygame.K_DELETE):
                self.cut((self.editing_pos, self.editing_pos+1))

            elif Input.get_key_down(pygame.K_LEFT):
                self.set_edit_pos(1, sub=True)
                self.bar_reset()  

            elif Input.get_key_down(pygame.K_RIGHT):
                self.set_edit_pos(1, add=True)
                self.bar_reset()  

            elif Input.get_key_down(pygame.K_KP_ENTER) or Input.get_key_down(13):
                self.focused = False
                self.input_event.invoke(self.text)

            elif Input.get_key(pygame.K_BACKSPACE):
                if self.wait_backspace.run_periodic_task():
                    self.toggle_backspace()

            elif Input.get_key_up(pygame.K_BACKSPACE):
                self.backspace = False

            if self.backspace:
                if self.backtime.run_periodic_task():
                    self.focus_cut(1)
        
        if not self.stay and Input.get_mouse_down(0):
            self.focused = False
        
        self.field.text = self.text + self.text_editing
        edit_text_pos = self.editing_pos + len(self.text_editing)
        
        if self.focused:
            if self.timertask.run_periodic_task():
                self.toggle_bar()
            
            if self.input_line.location.visible:
                pos = self.field.get_position(edit_text_pos) + self.field.location.world_position
                pos.x += 5
                pos.y += 40
                self.input_line.location.position = pos
            self.fake.location.visible = False
        else:
            self.input_line.location.visible = False
            if self.field.text == "":
                self.fake.location.visible = True

    def event(self, event):
        """입력 필드에 이벤트 직접 받기

        Args:
            event (event): pygame 에 이벤트입니다_
        """
        if self.focused:
            if event.type == pygame.TEXTEDITING:
                if len(self.text) < self.limit:
                    self.text_edit = True
                    self.text_editing = event.text
                    self.text_editing_pos = event.start
                    self.bar_reset()
            elif event.type == pygame.TEXTINPUT:
                self.text_editing = ""
                self.text_edit = False
                if len(self.text) < self.limit:
                    self.focus_insert(event.text)
                    self.bar_reset()

class InputLine(UI):
    def __init__(self, name, layer, tag, visible, position, rotation, parent_name, y):
        super().__init__(name, layer, tag, visible, position, rotation, parent_name)
        
        image = ImageObject(self, surface=(5, y), type="topleft", follow=True)
        image.og_image.fill((0, 0, 0))
        self.components.append(image)


import pygame
from pygamefwk.objects.component import Component
from pygamefwk.objects.components.soundListener import SoundListener

class SoundSource(Component):
    """소리를 재생, 정지하며
    거리에따른 음량조절을 할수있다."""
    def __init__(self, object, path, volume, vol_lambda, mode=None):
        self.vol_lambda = vol_lambda
        self.object = object
        self.volume = volume
        match mode:
            case None:
                self.mode = "sound"
                self.sound = pygame.mixer.Sound(path)
            case "endPlay":
                self.mode = "end"
                self.sound = path

    def set_volume(self):
        distance = self.object.location.world_position.distance_to(SoundListener.listener.object.location.world_position if SoundListener.listener != None else (0, 0))
        volume = self.vol_lambda(distance)

        if self.mode == "sound":
            self.sound.set_volume(volume + self.volume)
            
        elif self.mode == "end":
            pygame.mixer.music.set_volume(volume + self.volume)

    def play(self, loops=0, maxtime=0, fade_ms=0, start=0):
        distance = self.object.location.world_position.distance_to(SoundListener.listener.object.location.world_position if SoundListener.listener != None else (0, 0))
        volume = self.vol_lambda(distance)

        if self.mode == "sound":
            self.sound.set_volume(volume + self.volume)
            self.sound.play(loops, maxtime, fade_ms)
            
        elif self.mode == "end":
            pygame.mixer.music.stop()
            pygame.mixer.music.load(self.sound)
            pygame.mixer.music.set_volume(volume + self.volume)
            pygame.mixer.music.play(loops, start, fade_ms)
            
    def stop(self):
        if self.mode == "sound":
            self.sound.stop()
        elif self.mode == "end":
            pygame.mixer.music.stop()

class SoundManager(Component):
    def __init__(self, dict):
        self.dict = dict

    def play(self, name, loops=0):
        try:
            self.dict[name].play(loops=loops)
        except Exception as e:
            print(e, " 이름이 틀릴 가능성이 높음")

    def stop(self, name):
        try:
            self.dict[name].stop()
        except Exception as e:
            print(e, " 이름이 틀릴 가능성이 높음")

    def mixer_stop(self):
        pygame.mixer.music.stop()



# 편의성 import

__version__ = "1.0.0"

from pygamefwk.objects.component import Component

from pygamefwk.manger    import Manger
from pygamefwk           import game
from pygamefwk.game      import Game
from pygamefwk.objects   import *
from pygamefwk.sheet     import SurfaceSheet, TileSheet
from pygamefwk.timertask import TimerTask, OnceTimerTask
from pygamefwk.input     import Input
from pygamefwk.event     import Event
from pygamefwk.location  import Location
from pygamefwk import Vector

from pygame import (
    Surface,
    Rect
)

from pygame.constants import *



import inspect
from pygame import time
from typing import Callable
from pygamefwk.error import FunctionError

class TimerTask:
    """여러번 체크합니다"""
    def __init__(self, tick: int, event: Callable = lambda:None, *value, **kwargs):
        self.tick = tick
        self.last_update = 0
        if inspect.ismethod(event) and inspect.isfunction(event):
            raise FunctionError(f"입력받은 값은 는 함수가 아닙니다")
        self.event = event
        self.value = value
        self.kwargs = kwargs
        
    def not_update_run(self):
        """시간이 지났나 검사하기만 합니다"""
        if time.get_ticks() - self.last_update > self.tick:
            self.event(*self.value, **self.kwargs)
            return True
        return False

    def run_periodic_task(self):
        """시간이 지났나 검사, 리셋합니다"""
        if time.get_ticks() - self.last_update > self.tick:
            self.reset()
            self.event(*self.value, **self.kwargs)
            return True
        return False
    
    def reset(self):
        self.last_update = time.get_ticks()
        
class OnceTimerTask(TimerTask):
    """한번 체크하면 리셋할떄까지 체크를 하지 않습니다."""
    def __init__(self, tick, event=lambda:None, *value, **kwargs):
        super().__init__(tick, event, *value, **kwargs)
        self.once = False
    
    def run_periodic_task(self):
        if time.get_ticks() - self.last_update > self.tick and not self.once:
            self.event(*self.value, **self.kwargs)
            self.once = True
            return True
        return False
    
    def reset(self):
        super().reset()
        self.once = False
    


from pygamefwk.objects.object import Object
from pygamefwk.location import Parent

class ParentObject(Object):
    def __init__(self):
        super().__init__("parent", 0, "parent")
        self.visible = False
        self.location = Parent(self)
        
    def set_parent(self):
        pass


"""이미지를 시작 단계에서 미리 불러옴"""
from pygame import image, PixelArray, Surface
from pygame.transform import scale
from pygamefwk.objects.components.image import red

class SurfaceSheet:
    def __init__(self, name, is_hits, paths, default):
        self.name = name
        self.size = default
        cache = [image.load(path).convert_alpha() if path != None else None for path in paths]
        self.images = []
        for surf in cache:
            if surf == None: 
                self.images.append(None)
                continue

            xs, ys = surf.get_size()
            img = scale(surf, (xs*default, ys*default))
            if is_hits:
                red[img] = scale(get_hit_image(surf), (xs*default, ys*default))
            self.images.append(img)
    
class TileSheet:
    def __init__(self, name, is_hits, paths, default):
        self.name = name
        self.size = default
        cache = [image.load(path).convert_alpha() if path != None else None for path in paths]
        self.surfaces = []

        for mage in cache:
            if mage == None: 
                self.surfaces.append(None)
                continue

            img = scale(mage, (default, default))
            if is_hits:
                red[img] = scale(get_hit_image(mage), (default, default))
            self.surfaces.append(img)


def get_hit_image(s: Surface): # 매우 느립니다. 그래서 초기 시작부분에 미리 사용합니다.
    """이미지에 대해 상대적으로 빨갛게 물들입니다."""
    f = s.copy()
    f.lock()
    p = PixelArray(f)
    h=f.get_height()
    j=f.unmap_rgb
    for w in range(f.get_width()):
        l=p[w]
        for y in range(h):
            t = j(l[y])
            r = t.r + 100
            o = (r % 255) // 2
            t.r = r if r<255 else 255
            t.g = max(t.g-o,0)
            t.b = max(t.b-o,0)
            l[y] = t
    del p
    f.unlock()
    return f


from setuptools import setup, find_packages

setup(
    name='pygamefwk',
    version='1.0.0',
    description='pygame framework',
    author= 'fireing123',
    author_email= 'gimd82368@gmail.com',
    url='https://github.com/fireing123/pygamefwk',
    install_requires=["pygame"],
    packages=find_packages(),
    keywords=['pygame framework', 'fireing123', 'pygame'],
    long_description=open('README.md', 'r', encoding="UTF8").read(),
    long_description_content_type='text/markdown'
)



from pygamefwk import Vector
from typing import List, TYPE_CHECKING, Sequence

if TYPE_CHECKING:
    from pygamefwk.objects.gameobject import GameObject

class Parent:
    def __init__(self, object: 'GameObject'):
        self.object = object
        self.visible = True
        self.world_visible = True
        self.position = Vector(0, 0)
        self.world_position = Vector(0, 0)
        self.rotation = 0
        self.world_rotation = 0
        self.children: List['Location'] = []

    def set_world(self):
        for child in self.children:
            child.change_location()

class Location:
    """위치, 각도, visible 부모 자식 관계를 형성, 위치, 각도를 상대적으로 만듬"""
    def __init__(self, object: 'GameObject', position: Vector, rotation: int, visible: bool):
        self.object = object
        self.__visible: bool = visible
        self.__position = Vector(position)
        self.__rotation: int = rotation
        self.__world_position = Vector(0, 0)
        self.__world_rotation = 0
        self.__world_visible = False
        self.parent : Parent = None
        self.children: List['Location'] = []

    def translate(self, vector: Vector):
        self.position += vector

    def set_parent(self, parent: Parent):
        self.parent = parent
        parent.children.append(self)
    
    @property
    def visible(self):
        return self.__visible

    @visible.setter
    def visible(self, value: bool):
        self.__visible = value
        self.change_location()

    @property
    def position(self):
        return self.__position
    
    @position.setter
    def position(self, vector: Sequence):
        self.__position = Vector(vector)
        self.change_location()

    @property
    def rotation(self):
        return self.__rotation
    
    @rotation.setter
    def rotation(self, degree: int):
        if degree > 360:
            degree = degree // 360
        self.__rotation = degree
        self.change_location()
        
    @property
    def world_position(self):
        return self.__world_position
    
    @property
    def world_rotation(self):
        return self.__world_rotation
    
    @property
    def world_visible(self):
        return self.__world_visible

    def change_location(self):
        """상위 오브젝트의 Location 정보가 갱신되거나 객체의 정보가 변경되면 world_XXX 를 업데이트 하기 위해 정해짐"""
        self.__world_position: Vector = self.parent.world_position + self.__position.rotate(self.parent.rotation)
        self.__world_rotation: int = self.parent.world_rotation + self.__rotation
        self.__world_visible: bool = self.visible and self.parent.world_visible
        for child in self.children:
            child.change_location()


from typing import List, TYPE_CHECKING
from pygamefwk.objects.components.reset import on_reset

if TYPE_CHECKING:
    from pygamefwk.objects.components.image import ImageObject

collide_images: List[List['ImageObject']] = [[],[],[],[],[],[],[],[],[],[]]

def reset():
    for layer in collide_images:
        layer.clear()

on_reset.add_lisner(reset)


import pygame
from pygamefwk.objects.component import Component
from pygamefwk.objects.gameobject import GameObject
from pygamefwk.manger import Manger
from pygamefwk.vector import Vector
from pygamefwk.objects.components.collide_mouse import collide_images
from pygamefwk.objects.components.reset import on_reset

red = {}

def reset():
    red.clear()

on_reset.add_lisner(reset)

cache = {}

class ImageObject(Component):
    """오브젝트에 위치, 각도를 기준으로 이미지를 렌더링함"""
    def __init__(self, object: GameObject, **kwargs):
        self.object: GameObject = object
        self.visible = True
        self.og_image = None
        self.low_mode = None

        if 'low_mode' in kwargs:
            self.low_mode = kwargs['low_mode'] # name, xy, left args
            self.og_image = pygame.Surface((self.low_mode[1]), pygame.SRCALPHA).convert_alpha()
            getattr(pygame.draw, self.low_mode[0])(self.og_image, *self.low_mode[2:])


        if 'path' in kwargs:
            if kwargs['path'] not in cache:
                cache[kwargs['path']] = pygame.image.load(kwargs['path']).convert_alpha()

            self.og_image = cache[kwargs['path']]

        if 'surface' in kwargs:
            self.og_image = pygame.Surface(kwargs['surface'], pygame.SRCALPHA).convert_alpha()
        elif 'value' in kwargs:
            self.og_image = kwargs['value']

        if 'size' in kwargs:
            size = Vector(self.og_image.get_size())
            sizeup = Vector(kwargs['size'])
            size.x *= sizeup.x
            size.y *= sizeup.y
            self.og_image = pygame.transform.scale(self.og_image, size)

        self.collide = kwargs.get("collide", False)
        if self.collide:
            self.collide_state = False
            collide_images[object.layer].append(self)
        self.camera_staticable = kwargs.get("follow", False)
        self.flip = kwargs.get("filp", [False, False]) # 반전하면 True
        self.type = 'center' if kwargs.get('type') == None else kwargs['type']

        self.image = self.og_image

        if self.og_image != None:
            self.rect = self.image.get_rect()
        self.camera = Manger.scene.camera
        self.__cellophane = False

    def set_flip(self, value):
        if not (self.flip[0] == value):
            self.flip[0] = value

    def set_cellophane(self, value):
        if self.__cellophane != value:
            self.__cellophane = value

    def set_orginal_image(self, image):
        self.og_image = image

    def delete(self):
        try:
            if self.collide:
                collide_images[self.object.layer].remove(self)
        except: pass

    def render(self, surface, camera):
        if self.visible and self.og_image != None:
                
            image = red[self.og_image] if self.__cellophane else self.og_image # 만약 칠하면 og_image 주소를 키로 접근해 칠해진 image 를 얻음

            if self.camera_staticable: # 돌린 직사각형의 위치를 설정합니다
                rotated_rect = get_rotated_range(image.get_rect(), self.object.location.world_rotation)
                setattr(rotated_rect, self.type, camera.centerXY(self.object.render_position))
            else:
                rotated_rect = get_rotated_range(image.get_rect(), self.object.location.world_rotation + camera.location.world_rotation)
                setattr(rotated_rect, self.type, camera(self.object.render_position.floor_vector()))

            if rotated_rect.colliderect(0, 0, Manger.WIDTH, Manger.HEIGHT): # 화면안에 일부라도 있으면 실행합니다
                self.image = pygame.transform.flip(image, *self.flip)
                if self.camera_staticable: # 이미지를 실제도 돌립니다
                    self.image = pygame.transform.rotate(self.image, self.object.location.world_rotation)
                else:
                    self.image = pygame.transform.rotate(self.image, self.object.location.world_rotation + camera.location.world_rotation)

                if self.camera_staticable: # 이미지의 위치를 지정합니다
                    self.rect = self.image.get_rect(**{self.type:camera.centerXY(self.object.render_position.floor_vector())})
                else:
                    self.rect = self.image.get_rect(**{self.type:camera(self.object.render_position.floor_vector())})
                surface.blit(self.image, self.rect)

def get_rotated_range(rect: pygame.Rect, angle):
    """angle 각도로 돌린 직사각형과 외접하는 큰 직사각형을 반환합니다!"""
    if angle == 0:
        return rect
    
    cx, cy = rect.center

    corners = [
        rect.topleft,
        rect.topright,
        rect.bottomright,
        rect.bottomleft
    ]

    rotated_corners = [rotate_vector(x, y, cx, cy, angle) for (x, y) in corners]

    min_x = min(x for (x, y) in rotated_corners)
    max_x = max(x for (x, y) in rotated_corners)
    min_y = min(y for (x, y) in rotated_corners)
    max_y = max(y for (x, y) in rotated_corners)

    rotated_rect = pygame.Rect(min_x, min_y, max_x - min_x, max_y - min_y)

    return rotated_rect

def rotate_vector(x, y, rx, ry, angle):
    nvector = Vector(x - rx, y - ry).rotate(angle)
    return nvector.floor_vector()


from pygamefwk.objects.ui.ui import UI
from pygamefwk.event import Event
from pygamefwk.input import Input
from pygamefwk.objects.components.image import ImageObject

class Button(UI):
    def __init__(self, name, layer, tag, visible, position, rotation, parent_name, default, **kwargs):
        super().__init__(name, layer, tag, visible, position, rotation, parent_name)
        if default != None:
            kwargs["path"] = default
        self.image = ImageObject(self, follow=True, collide=True, **kwargs)
        self.components.append(self.image)

        self.dark = ImageObject(self, surface=self.image.og_image.get_size(), follow=True)
        self.components.append(self.dark)
        
        self.dark.og_image.fill((0,0,0,20))
        self.dark.visible = False

        self.rect = self.image.rect
        self.is_click = Event()
    
    def on_mouse_stay(self, pos):
        if Input.get_mouse_down(0):
            self.dark.visible = True
        elif Input.get_mouse_up(0):
            if self.dark.visible:
                self.is_click.invoke()
            self.dark.visible = False
    
    def on_mouse_exit(self, pos: tuple[int, int]):
        self.dark.visible = False